1 REMark Transfonter
2 REMark This program creates a Sinclair QL ISO 8859-1 font.
3 REMark It is written in SBASIC.
4 REMark It has been tested with the QPC2 QL emulator
5 REMark running on Debian GNU/Linux with Wine.
6 REMark Copyright (C) 2010 Marcos Cruz (http://programandala.net)
7 REMark License: http://programandala.net/license
8 REMark ---------------- Changelog
9 REMark 2010-08-21
10 REMark * First draft.
11 REMark 2010-08-22
12 REMark * First working version.
13 REMark 2010-08-23
14 REMark * New system: patterns are taken from several input fonts.
15 REMark 2010-08-24
16 REMark * All missing characters are defined.
17 REMark 2010-08-26
18 REMark * Some character names are fixed.
19 REMark * Bug fixed: now the pattern for undefined characters is a constant, not peeked from one of the input fonts (what worked only with the old system, with only one input font).
20 REMark * Bug fixed: now input_font_address is set to zero after releasing its space; this makes it possible to run the program more than once, otherwise an error was reported.
21 REMark ---------------- Global
22 LET char_bytes = 9 : REMark Bytes per char pattern.
23 DIM pattern$(char_bytes-1,8) : REMark Temporary store for patterns, in strings of 8-bit binary values.
24 LET undefined_pattern$ = "T(T(T(T(T" : REMark Checkered pattern used for undefined chars.
25 LET input_fonts_dir$ = "win1_prg_font_editor_" : REMark Path of all input fonts.
26 REMark ---------------- Main
27 create_iso_8859_1_font "dos1_fonts_iso_8859-1_font"
28 STOP
29 REMark ---------------- Font manipulation
30 DEFine PROCedure open_input_font(input_font$)
31   IF input_font_address : close_input_font
32   LET input_font_length = FLEN(\input_fonts_dir$&input_font$)
33   LET input_font_address = ALCHP(input_font_length)
34   LBYTES input_fonts_dir$&input_font$,input_font_address
35   LET input_first_char = first_char_in_font(input_font_address)
36   LET input_chars = chars_in_font(input_font_address)
37   LET input_last_char = input_first_char+input_chars-1
38 END DEFine
39 DEFine PROCedure open_output_font
40   LET output_first_char = 32
41   LET output_last_char = 255
42   LET output_chars = output_last_char-output_first_char+1
43   LET output_font_length = 2+char_bytes+output_chars*char_bytes
44   LET output_font_address = ALCHP(output_font_length)
45   create_output_font_header
46 END DEFine
47 DEFine PROCedure create_output_font_header
48   POKE output_font_address,output_first_char-1
49   POKE output_font_address+1,output_chars
50   POKE$ output_font_address+2,undefined_pattern$
51 END DEFine
52 DEFine PROCedure close_input_font
53   RECHP input_font_address
54   LET input_font_address = 0
55 END DEFine
56 DEFine PROCedure save_output_font(output_font$)
57   SBYTES_O output_font$,output_font_address,output_font_length
58   RECHP output_font_address
59 END DEFine
60 REMark ---------------- Char manipulation
61 DEFine PROCedure undefined_chars(first_char,last_char)
62   REMark Copy into the specified range of chars of the output font
63   REMark the undefined pattern.
64   LOCal char
65   FOR char = first_char TO last_char
66     POKE$ output_char_address(char),undefined_pattern$
67   END FOR char
68 END DEFine
69 DEFine PROCedure copy_chars(first_char,last_char)
70   REMark Copy a range of chars from the input font into the same position of the output font.
71   LOCal char
72   FOR char = first_char TO last_char
73     copy_char char,char
74   END FOR char
75 END DEFine
76 DEFine PROCedure copy_char(input_char,output_char)
77   REMark Copy one char from the input font into the output font.
78   POKE$ output_char_address(output_char),PEEK$(input_char_address(input_char),char_bytes)
79 END DEFine
80 DEFine PROCedure pattern_to_char(char)
81   LOCal char_base_address
82   LOCal row
83   LET char_base_address = output_char_address(char)
84   FOR row=0 TO char_bytes-1
85     POKE char_base_address+row,BIN(pattern$(row))
86   END FOR row
87 END DEFine
88 DEFine FuNction input_char_address(char)
89   REMark Return the address of a char of the input font.
90   RETurn char_address(input_font_address,char)
91 END DEFine
92 DEFine FuNction output_char_address(char)
93   REMark Return the address of a char of the output font.
94   RETurn char_address(output_font_address,char)
95 END DEFine
96 REMark ---------------- Font information
97 DEFine FuNction char_address(font_address,char)
98   REMark Return the address of a char in a font.
99   RETurn font_address + 2 + char_bytes + char_bytes*(char-first_char_in_font(font_address))
100 END DEFine
101 DEFine FuNction first_char_in_font(font_address)
102   REMark Return the code of the first char in a font.
103   RETurn PEEK(font_address)+1
104 END DEFine
105 DEFine FuNction chars_in_font(font_address)
106   REMark Return the number of chars in a font.
107   RETurn PEEK(font_address+1)
108 END DEFine
109 REMark ---------------- Font translation
110 DEFine PROCedure create_iso_8859_1_font(output_font$)
111   REMark Create an ISO 8859-1 font
112   REMark by getting character patterns from other fonts.
113   REMark The fonts used are bundled with Font Editor, by Dilwyn Jones
114   REMark (http://www.dilwyn.me.uk/fonts/).
115   REMark References on the Internet:
116   REMark http://en.wikipedia.org/wiki/ISO/IEC_8859-1
117   REMark http://ciberia.ya.com/rulo_sinclairql/charset.html
118   open_output_font
119   open_input_font "qlfont1_font" : REMark QL font 1 (chars 32-127)
120   copy_chars 32,126 : REMark ASCII
121   undefined_chars 127,255 : REMark the rest is undefined by default
122   copy_char 32,160 : REMark no-break space (NBSP)
123   copy_char 45,173 : REMark soft hyphen (SHY)
124   copy_char 96,163 : REMark pound sign
125   copy_char 127,169 : REMark copyright sign
126   open_input_font "qlfont2_font" : REMark QL font 2 (chars 128-191)
127   copy_char 128,228 : REMark a with diaeresis
128   copy_char 129,227 : REMark a tilde
129   copy_char 130,229 : REMark a with ring
130   copy_char 131,233 : REMark e with acute
131   copy_char 132,246 : REMark o with diaeresis
132   copy_char 133,245 : REMark o tilde
133   copy_char 134,248 : REMark o with stroke
134   copy_char 135,252 : REMark u with diaeresis
135   copy_char 136,231 : REMark c with cedilla
136   copy_char 137,241 : REMark n tilde
137   copy_char 138,230 : REMark ae
138   REMark char 139 (oe) doesn't exist in ISO 8859-1
139   copy_char 140,225 : REMark a with acute
140   copy_char 141,224 : REMark a with grave
141   copy_char 142,226 : REMark a with circumflex
142   copy_char 143,235 : REMark e with diaeresis
143   copy_char 144,232 : REMark e with grave
144   copy_char 145,234 : REMark e with circumflex
145   copy_char 146,239 : REMark i with diaeresis
146   copy_char 147,237 : REMark i with acute
147   copy_char 148,236 : REMark i with grave
148   copy_char 149,238 : REMark i with circumflex
149   copy_char 150,243 : REMark o with acute
150   copy_char 151,242 : REMark o with grave
151   copy_char 152,244 : REMark o with circumflex
152   copy_char 153,250 : REMark u with acute
153   copy_char 154,249 : REMark u with grave
154   copy_char 155,251 : REMark u with circumflex
155   copy_char 156,223 : REMark sharp s
156   copy_char 157,162 : REMark cent sign
157   copy_char 158,165 : REMark yen sign
158   copy_char 159,96 : REMark grave accent
159   copy_char 160,196 : REMark A with diaeresis
160   copy_char 161,195 : REMark A tilde
161   copy_char 162,197 : REMark A with ring
162   copy_char 163,201 : REMark E with acute
163   copy_char 164,214 : REMark O with diaeresis
164   copy_char 165,213 : REMark O tilde
165   copy_char 166,216 : REMark O with stroke
166   copy_char 167,220 : REMark U with diaeresis
167   copy_char 168,199 : REMark C with cedilla
168   copy_char 169,209 : REMark N tilde
169   copy_char 170,198 : REMark AE
170   REMark char 171 (OE) doesn't exist in ISO 8859-1
171   REMark chars 172-175 (greek letters) don't exist in ISO 8859-1
172   copy_char 176,181 : REMark micro sign
173   REMark chars 177-178 (greek letters) don't exist in ISO 8859-1
174   copy_char 179,161 : REMark inverted exclamation mark
175   copy_char 180,191 : REMark inverted question mark
176   REMark char 181 doesn't exist in ISO 8859-1
177   copy_char 182,167 : REMark section sign
178   copy_char 183,164 : REMark currency sign
179   copy_char 184,171 : REMark left-pointing double angle quotation mark
180   copy_char 185,187 : REMark right-pointing double angle quotation mark
181   copy_char 186,176 : REMark degree sign
182   copy_char 187,247 : REMark division sign
183   REMark chars 188-191 (arrows) don't exist in ISO 8859-1
184   open_input_font "pc8_combined_font" : REMark Code Page 437 font, by Dilwyn Jones
185   copy_char 152,255 : REMark y with diaeresis
186   copy_char 166,170 : REMark feminine ordinal indicator
187   copy_char 167,186 : REMark masculine ordinal indicator
188   copy_char 169,172 : REMark not sign
189   copy_char 171,189 : REMark 1/2
190   copy_char 172,188 : REMark 1/4
191   copy_char 241,177 : REMark plus-minus sign
192   REMark copy_char 248,176 : rem degree sign (already in QL font, character 186)
193   REMark copy_char 250,183 : rem middle dot (it doesn't look fine; it will be redefined)
194   copy_char 253,178 : REMark superscript 2
195   REMark Missing chars have to be defined
196   REMark 166, broken vertical bar
197   LET pattern$(0)="00010000"
198   LET pattern$(1)="00010000"
199   LET pattern$(2)="00010000"
200   LET pattern$(3)="00000000"
201   LET pattern$(4)="00010000"
202   LET pattern$(5)="00010000"
203   LET pattern$(6)="00010000"
204   LET pattern$(7)="00000000"
205   LET pattern$(8)="00000000"
206   pattern_to_char 166
207   REMark 168, diaeresis
208   LET pattern$(0)="01000100"
209   LET pattern$(1)="00000000"
210   LET pattern$(2)="00000000"
211   LET pattern$(3)="00000000"
212   LET pattern$(4)="00000000"
213   LET pattern$(5)="00000000"
214   LET pattern$(6)="00000000"
215   LET pattern$(7)="00000000"
216   LET pattern$(8)="00000000"
217   pattern_to_char 168
218   REMark 174, registered sign
219   LET pattern$(0)="00111000"
220   LET pattern$(1)="01000100"
221   LET pattern$(2)="01110100"
222   LET pattern$(3)="01001100"
223   LET pattern$(4)="01110100"
224   LET pattern$(5)="01001100"
225   LET pattern$(6)="00111000"
226   LET pattern$(7)="00000000"
227   LET pattern$(8)="00000000"
228   pattern_to_char 174
229   REMark 175, macron
230   LET pattern$(0)="01111100"
231   LET pattern$(1)="00000000"
232   LET pattern$(2)="00000000"
233   LET pattern$(3)="00000000"
234   LET pattern$(4)="00000000"
235   LET pattern$(5)="00000000"
236   LET pattern$(6)="00000000"
237   LET pattern$(7)="00000000"
238   LET pattern$(8)="00000000"
239   pattern_to_char 175
240   REMark 179, superscript 3
241   REMark first version
242   LET pattern$(0)="01100000"
243   LET pattern$(1)="00010000"
244   LET pattern$(2)="01100000"
245   LET pattern$(3)="00010000"
246   LET pattern$(4)="01100000"
247   LET pattern$(5)="00000000"
248   LET pattern$(6)="00000000"
249   LET pattern$(7)="00000000"
250   LET pattern$(8)="00000000"
251   REMark second version
252   LET pattern$(0)="01110000"
253   LET pattern$(1)="00110000"
254   LET pattern$(2)="00010000"
255   LET pattern$(3)="01110000"
256   LET pattern$(4)="00000000"
257   LET pattern$(5)="00000000"
258   LET pattern$(6)="00000000"
259   LET pattern$(7)="00000000"
260   LET pattern$(8)="00000000"
261   pattern_to_char 179
262   REMark 180, acute accent
263   LET pattern$(0)="00001000"
264   LET pattern$(1)="00010000"
265   LET pattern$(2)="00000000"
266   LET pattern$(3)="00000000"
267   LET pattern$(4)="00000000"
268   LET pattern$(5)="00000000"
269   LET pattern$(6)="00000000"
270   LET pattern$(7)="00000000"
271   LET pattern$(8)="00000000"
272   pattern_to_char 180
273   REMark 182, paragraph sign / pilcrow sign
274   LET pattern$(0)="00111100"
275   LET pattern$(1)="01110100"
276   LET pattern$(2)="01110100"
277   LET pattern$(3)="00110100"
278   LET pattern$(4)="00010100"
279   LET pattern$(5)="00010100"
280   LET pattern$(6)="00010100"
281   LET pattern$(7)="00000000"
282   LET pattern$(8)="00000000"
283   pattern_to_char 182
284   REMark 183, middle dot
285   LET pattern$(0)="00000000"
286   LET pattern$(1)="00000000"
287   LET pattern$(2)="00000000"
288   LET pattern$(3)="00011000"
289   LET pattern$(4)="00011000"
290   LET pattern$(5)="00000000"
291   LET pattern$(6)="00000000"
292   LET pattern$(7)="00000000"
293   LET pattern$(8)="00000000"
294   pattern_to_char 183
295   REMark 184, cedilla
296   LET pattern$(0)="00000000"
297   LET pattern$(1)="00000000"
298   LET pattern$(2)="00000000"
299   LET pattern$(3)="00000000"
300   LET pattern$(4)="00000000"
301   LET pattern$(5)="00000000"
302   LET pattern$(6)="00000000"
303   LET pattern$(7)="00010000"
304   LET pattern$(8)="00100000"
305   pattern_to_char 184
306   REMark 185, superscript 1
307   REMark first version
308   LET pattern$(0)="00110000"
309   LET pattern$(1)="01010000"
310   LET pattern$(2)="00010000"
311   LET pattern$(3)="00010000"
312   LET pattern$(4)="00010000"
313   LET pattern$(5)="00000000"
314   LET pattern$(6)="00000000"
315   LET pattern$(7)="00000000"
316   LET pattern$(8)="00000000"
317   REMark second version
318   LET pattern$(0)="00100000"
319   LET pattern$(1)="01100000"
320   LET pattern$(2)="00100000"
321   LET pattern$(3)="01110000"
322   LET pattern$(4)="00000000"
323   LET pattern$(5)="00000000"
324   LET pattern$(6)="00000000"
325   LET pattern$(7)="00000000"
326   LET pattern$(8)="00000000"
327   pattern_to_char 185
328   REMark 190, 3/4
329   REMark first version
330   LET pattern$(0)="01110000"
331   LET pattern$(1)="00110000"
332   LET pattern$(2)="00010000"
333   LET pattern$(3)="01111000"
334   LET pattern$(4)="00010000"
335   LET pattern$(5)="00101000"
336   LET pattern$(6)="01011000"
337   LET pattern$(7)="00111100"
338   LET pattern$(8)="00001000"
339   REMark second version
340   LET pattern$(0)="01110000"
341   LET pattern$(1)="00110000"
342   LET pattern$(2)="00011000"
343   LET pattern$(3)="01110000"
344   LET pattern$(4)="00101000"
345   LET pattern$(5)="01011000"
346   LET pattern$(6)="00111100"
347   LET pattern$(7)="00001000"
348   LET pattern$(8)="00000000"
349   pattern_to_char 190
350   REMark 192, A with grave
351   LET pattern$(0)="00100000"
352   LET pattern$(1)="00010000"
353   LET pattern$(2)="00101000"
354   LET pattern$(3)="01000100"
355   LET pattern$(4)="01111100"
356   LET pattern$(5)="01000100"
357   LET pattern$(6)="01000100"
358   LET pattern$(7)="00000000"
359   LET pattern$(8)="00000000"
360   pattern_to_char 192
361   REMark 193, A with acute
362   LET pattern$(0)="00001000"
363   LET pattern$(1)="00010000"
364   LET pattern$(2)="00101000"
365   LET pattern$(3)="01000100"
366   LET pattern$(4)="01111100"
367   LET pattern$(5)="01000100"
368   LET pattern$(6)="01000100"
369   LET pattern$(7)="00000000"
370   LET pattern$(8)="00000000"
371   pattern_to_char 193
372   REMark 194, A with circumflex
373   LET pattern$(0)="00010000"
374   LET pattern$(1)="00101000"
375   LET pattern$(2)="00000000"
376   LET pattern$(3)="00111000"
377   LET pattern$(4)="01000100"
378   LET pattern$(5)="01111100"
379   LET pattern$(6)="01000100"
380   LET pattern$(7)="00000000"
381   LET pattern$(8)="00000000"
382   pattern_to_char 194
383   REMark 200, E with grave
384   LET pattern$(0)="00100000"
385   LET pattern$(1)="00010000"
386   LET pattern$(2)="01111100"
387   LET pattern$(3)="01000000"
388   LET pattern$(4)="01111100"
389   LET pattern$(5)="01000000"
390   LET pattern$(6)="01111100"
391   LET pattern$(7)="00000000"
392   LET pattern$(8)="00000000"
393   pattern_to_char 200
394   REMark 202, E with circumflex
395   LET pattern$(0)="00010000"
396   LET pattern$(1)="00101000"
397   LET pattern$(2)="01111100"
398   LET pattern$(3)="01000000"
399   LET pattern$(4)="01111100"
400   LET pattern$(5)="01000000"
401   LET pattern$(6)="01111100"
402   LET pattern$(7)="00000000"
403   LET pattern$(8)="00000000"
404   pattern_to_char 202
405   REMark 203, E with diaeresis
406   LET pattern$(0)="01000100"
407   LET pattern$(1)="00000000"
408   LET pattern$(2)="01111100"
409   LET pattern$(3)="01000000"
410   LET pattern$(4)="01111100"
411   LET pattern$(5)="01000000"
412   LET pattern$(6)="01111100"
413   LET pattern$(7)="00000000"
414   LET pattern$(8)="00000000"
415   pattern_to_char 203
416   REMark 204, I with grave
417   LET pattern$(0)="00100000"
418   LET pattern$(1)="00010000"
419   LET pattern$(2)="00111000"
420   LET pattern$(3)="00010000"
421   LET pattern$(4)="00010000"
422   LET pattern$(5)="00010000"
423   LET pattern$(6)="00111000"
424   LET pattern$(7)="00000000"
425   LET pattern$(8)="00000000"
426   pattern_to_char 204
427   REMark 205, I with acute
428   LET pattern$(0)="00001000"
429   LET pattern$(1)="00010000"
430   LET pattern$(2)="00111000"
431   LET pattern$(3)="00010000"
432   LET pattern$(4)="00010000"
433   LET pattern$(5)="00010000"
434   LET pattern$(6)="00111000"
435   LET pattern$(7)="00000000"
436   LET pattern$(8)="00000000"
437   pattern_to_char 205
438   REMark 206, I with circumflex
439   LET pattern$(0)="00010000"
440   LET pattern$(1)="00101000"
441   LET pattern$(2)="00000000"
442   LET pattern$(3)="00111000"
443   LET pattern$(4)="00010000"
444   LET pattern$(5)="00010000"
445   LET pattern$(6)="00111000"
446   LET pattern$(7)="00000000"
447   LET pattern$(8)="00000000"
448   pattern_to_char 206
449   REMark 207, I with diaeresis
450   LET pattern$(0)="00100100"
451   LET pattern$(1)="00000000"
452   LET pattern$(2)="00111000"
453   LET pattern$(3)="00010000"
454   LET pattern$(4)="00010000"
455   LET pattern$(5)="00010000"
456   LET pattern$(6)="00111000"
457   LET pattern$(7)="00000000"
458   LET pattern$(8)="00000000"
459   pattern_to_char 207
460   REMark 208, capital Eth
461   LET pattern$(0)="00111000"
462   LET pattern$(1)="00100100"
463   LET pattern$(2)="00100100"
464   LET pattern$(3)="01110100"
465   LET pattern$(4)="00100100"
466   LET pattern$(5)="00100100"
467   LET pattern$(6)="00111000"
468   LET pattern$(7)="00000000"
469   LET pattern$(8)="00000000"
470   pattern_to_char 208
471   REMark 210, O with grave
472   LET pattern$(0)="00100000"
473   LET pattern$(1)="00010000"
474   LET pattern$(2)="00111000"
475   LET pattern$(3)="01000100"
476   LET pattern$(4)="01000100"
477   LET pattern$(5)="01000100"
478   LET pattern$(6)="00111000"
479   LET pattern$(7)="00000000"
480   LET pattern$(8)="00000000"
481   pattern_to_char 210
482   REMark 211, O with acute
483   LET pattern$(0)="00001000"
484   LET pattern$(1)="00010000"
485   LET pattern$(2)="00111000"
486   LET pattern$(3)="01000100"
487   LET pattern$(4)="01000100"
488   LET pattern$(5)="01000100"
489   LET pattern$(6)="00111000"
490   LET pattern$(7)="00000000"
491   LET pattern$(8)="00000000"
492   pattern_to_char 211
493   REMark 211, O with circumflex
494   LET pattern$(0)="00010000"
495   LET pattern$(1)="00101000"
496   LET pattern$(2)="00111000"
497   LET pattern$(3)="01000100"
498   LET pattern$(4)="01000100"
499   LET pattern$(5)="01000100"
500   LET pattern$(6)="00111000"
501   LET pattern$(7)="00000000"
502   LET pattern$(8)="00000000"
503   pattern_to_char 212
504   REMark 215, multiplication sign
505   LET pattern$(0)="00000000"
506   LET pattern$(1)="01000100"
507   LET pattern$(2)="00101000"
508   LET pattern$(3)="00010000"
509   LET pattern$(4)="00101000"
510   LET pattern$(5)="01000100"
511   LET pattern$(6)="00000000"
512   LET pattern$(7)="00000000"
513   LET pattern$(8)="00000000"
514   pattern_to_char 215
515   REMark 217, U with grave
516   LET pattern$(0)="00100000"
517   LET pattern$(1)="00010000"
518   LET pattern$(2)="01000100"
519   LET pattern$(3)="01000100"
520   LET pattern$(4)="01000100"
521   LET pattern$(5)="01000100"
522   LET pattern$(6)="00111000"
523   LET pattern$(7)="00000000"
524   LET pattern$(8)="00000000"
525   pattern_to_char 217
526   REMark 218, U with acute
527   LET pattern$(0)="00001000"
528   LET pattern$(1)="00010000"
529   LET pattern$(2)="01000100"
530   LET pattern$(3)="01000100"
531   LET pattern$(4)="01000100"
532   LET pattern$(5)="01000100"
533   LET pattern$(6)="00111000"
534   LET pattern$(7)="00000000"
535   LET pattern$(8)="00000000"
536   pattern_to_char 218
537   REMark 219, U with circumflex
538   LET pattern$(0)="00010000"
539   LET pattern$(1)="00101000"
540   LET pattern$(2)="01000100"
541   LET pattern$(3)="01000100"
542   LET pattern$(4)="01000100"
543   LET pattern$(5)="01000100"
544   LET pattern$(6)="00111000"
545   LET pattern$(7)="00000000"
546   LET pattern$(8)="00000000"
547   pattern_to_char 219
548   REMark 221, Y with acute
549   LET pattern$(0)="00001000"
550   LET pattern$(1)="01010100"
551   LET pattern$(2)="01000100"
552   LET pattern$(3)="00101000"
553   LET pattern$(4)="00010000"
554   LET pattern$(5)="00010000"
555   LET pattern$(6)="00010000"
556   LET pattern$(7)="00000000"
557   LET pattern$(8)="00000000"
558   pattern_to_char 221
559   REMark 222, capital Thorn
560   LET pattern$(0)="00000000"
561   LET pattern$(1)="01000000"
562   LET pattern$(2)="01111000"
563   LET pattern$(3)="01000100"
564   LET pattern$(4)="01000100"
565   LET pattern$(5)="01111000"
566   LET pattern$(6)="01000000"
567   LET pattern$(7)="00000000"
568   LET pattern$(8)="00000000"
569   pattern_to_char 222
570   REMark 240, small Eth
571   LET pattern$(0)="00100000"
572   LET pattern$(1)="01111000"
573   LET pattern$(2)="00010000"
574   LET pattern$(3)="00111000"
575   LET pattern$(4)="01000100"
576   LET pattern$(5)="01000100"
577   LET pattern$(6)="00111000"
578   LET pattern$(7)="00000000"
579   LET pattern$(8)="00000000"
580   pattern_to_char 240
581   REMark 253, y with acute
582   LET pattern$(0)="00001000"
583   LET pattern$(1)="00010000"
584   LET pattern$(2)="01000100"
585   LET pattern$(3)="01000100"
586   LET pattern$(4)="01000100"
587   LET pattern$(5)="01000100"
588   LET pattern$(6)="00111100"
589   LET pattern$(7)="00000100"
590   LET pattern$(8)="00111000"
591   pattern_to_char 253
592   REMark 254, small Thorn
593   LET pattern$(0)="00000000"
594   LET pattern$(1)="01000000"
595   LET pattern$(2)="01000000"
596   LET pattern$(3)="01111000"
597   LET pattern$(4)="01000100"
598   LET pattern$(5)="01000100"
599   LET pattern$(6)="01111000"
600   LET pattern$(7)="01000000"
601   LET pattern$(8)="01000000"
602   pattern_to_char 254
603   close_input_font
604   show_output_font
605   save_output_font output_font$
606 END DEFine
607 REMark ---------------- Show the fonts
608 DEFine PROCedure show_output_font
609   PRINT output_font$
610   show_font output_font_address
611 END DEFine
612 DEFine PROCedure show_input_font
613   REMark Used while debugging.
614   show_font input_font_address
615 END DEFine
616 DEFine PROCedure show_font(font_address)
617   LOCal column,row,char
618   CHAR_USE font_address,0
619   PRINT
620   FOR column = 0 TO 15
621     PRINT TO 2;HEX$(column,4);
622   END FOR column
623   PRINT\\
624   FOR row = 0 TO 15
625     PRINT HEX$(row,4);
626     FOR column = 0 TO 15
627       LET char = row*16+column
628       IF char=10
629         CHAR_USE 0,0
630         PRINT TO 2;CHR$(0);
631         CHAR_USE font_address,0
632       ELSE
633         PRINT TO 2;CHR$(char);
634       END IF
635     END FOR column
636     PRINT
637   END FOR row
638   CHAR_USE 0,0
639 END DEFine
